/****************************************************************************
 * Copyright  2006 Altera Corporation, San Jose, California, USA.           *
 * All rights reserved. All use of this software and documentation is        *
 * subject to the License Agreement located at the end of this file below.   *
 ****************************************************************************/
/*****************************************************************************
*  File: small_boot_copier.s
*
*  This is an example of a small-code-size Nios II boot copier implemented 
*  in assembly language.  This program uses only Nios II CPU registers for 
*  data storage, no variables are stored in RAM, making this code completely 
*  relocatable.  
*
*  This boot copier can be build using the accompianing Makefile provided
*  with the example.  To build the boot copier, open a Nios II Command Shell 
*  in this directory and type "make all".  Type "make help" for more 
*  information
*
*  Feel free to customize this boot copier, but do so at your own risk.  This 
*  boot copier can only be supported by Altera in its current, unmodified form
*  
*****************************************************************************/

#include "small_boot_copier.h"

// This is the signature we expect to find at offset 0x0 of the boot image
#define VALID_BOOT_SIGNATURE 0xA5A5A5A5

// FLASH_BASE and BOOT_IMAGE_OFFSET are passed in through 'make'
#define BOOT_IMAGE_ADDR  ( FLASH_BASE + BOOT_IMAGE_OFFSET )

//  These all might be aliases for our boot copier's 
//  entry point. 

  .global reset
  .global _start
  .global main
  .global end_of_boot_copier


/*****************************************************************************
*  Function: main
*
*  Purpose: This is our boot copier's entry point. This function checks for
*  the presence of a applicatoin boot image in flash memory, copies it to its 
*  execution memory (set in Nios II IDE), then branches to the application's
*  entry point.  It's important to ensure all function calls are only one
*  level deep, since we are using one register to store the return pointer, 
*  not the stack.
*
*****************************************************************************/
reset:
_start:
main:

  // Clear the CPU's status-register, thereby disabling interrupts.
  wrctl   status, zero

  // Flush the instruction cache.
  movhi   counter,%hi(0x10000)
cache_flush_loop:
  initi   counter
  addi    counter, counter,-32
  bne     counter, zero, cache_flush_loop

  // Flush the pipeline
  flushp
    
  // Load the Boot Image address
  movhi   flash_ptr, %hi(BOOT_IMAGE_ADDR)
  ori     flash_ptr, flash_ptr, %lo(BOOT_IMAGE_ADDR)

  // Check the boot image signature
  nextpc  return_address_less_4
  br      read_32b_from_flash
  mov     temp_data, return_value
  movhi   expected_value, %hi(VALID_BOOT_SIGNATURE)
  ori     expected_value, expected_value, %lo(VALID_BOOT_SIGNATURE)
  bne     temp_data, expected_value, error
  
  // Now we can begin copying the boot record to RAM.  This loop
  // reads each individual record in the boot record, and copies it
  // to the appropriate location in RAM.  If the record is a JUMP
  // record, the loop breaks and branches to the address indicated
  // in the JUMP record.

  // Increment flash_ptr by 28 to skip past the boot image header and into
  // the real boot record (we already incremented it by 4 when we read the
  // signature)
  //
  // You may wish to do more with the header information than just skip
  // past it.  If so, this is the place to do that.
  addi    flash_ptr, flash_ptr, 28

  // Pre-load the expected halt value, 0xFFFFFFFF so we can compare
  // records to it later
  subi    expected_value, zero, 1

once_per_record_loop:

  // Read the first 4 bytes from the boot record (the length)
  nextpc  return_address_less_4
  br      read_32b_from_flash
  mov     boot_record_length, return_value

  // Read the next 4 bytes from the boot record (the destination address)
  nextpc  return_address_less_4
  br      read_32b_from_flash
  mov     boot_record_dest_addr, return_value

  // Test to see if boot_record_length is zero
  // If it is, this is a JUMP record so we need to jump.
  beq     boot_record_length, zero, jump_to_application

  // Test to see if boot_record_length is 0xFFFFFFFF
  // If it is, this is a HALT record.  We must halt
  beq     boot_record_length, expected_value, halt

  // If it's not a jump record or a halt record, it means we are
  // supposed to copy it to RAM
  nextpc  return_address_less_4
  br      copy_data_from_flash

  // When we get here, we're done with the current record.
  // And since it wasnt a JUMP or HALT record, we know there's
  // at least one more.  So we go back to process another one.
  br      once_per_record_loop


/*****************************************************************************
*  Function: jump_to_application
*
*  Purpose: This function jumps to the entry point of the application, stored
*  in "boot_record_dest_addr"
*
*****************************************************************************/
jump_to_application:

  // We must have found a jump-record. "boot_record_dest_addr" now holds 
  // the entry-point of the application in RAm, so we'll just call it.

  callr   boot_record_dest_addr

 // This is where we end up if the application ever returns.
loop_forever:
  br      loop_forever


/*****************************************************************************
*  Function: error
*
*  Purpose: This function is a simple error trap.  You may wish to add code
*  here which does something intelligent.
*
*****************************************************************************/
error:
	br error

/*****************************************************************************
*  Function: halt
*
*  Purpose: This function is called when a HALT record is found in the boot
*  record.  We halt execution by looping forever.
*
*****************************************************************************/
halt:
	br halt


/*****************************************************************************
*  Function: read_32b_from_flash
*
*  Purpose: Reads 4 bytes from flash and returns them as a 32-bit word.  The
*  bytes do not neccessarily need to fall on a 32-bit word boundary.
*
*  Registers used:
*       flash_ptr             - source address of the read
*       temp_data             - local data variable
*       byte_count            - number of bytes left to read
*       return_value          - 32-bit data which was read
*       return_address        - address of instruction following that 
*                               which called us
*
*****************************************************************************/
read_32b_from_flash:
	// Get the correct return-address
  addi    return_address, return_address_less_4, 4

  // Clear the return value
  mov     return_value, zero

  // Number of bytes to retrieve.
  movi    byte_count, 4

read_32b_loop:
  // Retrieve a byte and increment the flash pointer
  ldbuio  temp_data, 0(flash_ptr)
  addi    flash_ptr, flash_ptr, 1

  // OR it into the LSB of the result
  or      return_value, return_value, temp_data

  // Rotate the result left so that the latest byte is in the MSB.
  // This accounts for the little-endian byte order
  roli    return_value, return_value, 24

  // Decrement the counter, and loop
  subi    byte_count, byte_count, 1
  bne     byte_count, zero, read_32b_loop

  // Return.
  jmp     return_address


/*****************************************************************************
*  Function: copy_data_from_flash
*
*  Purpose: Copies a block of data from flash to RAM.
*
*  Registers used:
*       flash_ptr             - source address of the copy
*       boot_record_dest_addr - destination address of the copy
*       boot_record_length    - number of bytes to copy
*       temp_data             - local data variable
*       last_dest_address     - indicates the end of the copy
*       return_address        - address of instruction following that 
*                               which called us
*
*****************************************************************************/
copy_data_from_flash:
  // Get the correct return-address
  addi    return_address, return_address_less_4, 4

  // Calculate the last destination address for this record
  add     last_dest_address, boot_record_length, boot_record_dest_addr
  subi    last_dest_address, last_dest_address, 1
	
flash_copy_loop:
  ldbuio  temp_data, 0(flash_ptr)
  addi    flash_ptr, flash_ptr, 1
  stbio   temp_data, 0(boot_record_dest_addr)
  addi    boot_record_dest_addr, boot_record_dest_addr, 1

  // loop until the destination == 1 + the ending address
  ble     boot_record_dest_addr, last_dest_address, flash_copy_loop
		
  // Return
  jmp     return_address   

  .end


/*
 * This LED routine can be used for lighting LEDs for debug purposes
 * Since we only support one-level function calls, it's best to just 
 * copy and past this wherever you need it.  You'll also need to 
 * define LED_ADDR as the base address of your LED PIO.
 */
/* 
  // Debugging LED routine
  movhi   led_addr, %hi(LED_ADDR)
  ori     led_addr, led_addr, %lo(LED_ADDR)
  mov    led_value, return_address
  srli    led_value, led_value, 16
  stbio   led_value, 0(led_addr)
stall_loop:
	br stall_loop
*/

// end of file

/******************************************************************************
 *                                                                             *
 * License Agreement                                                           *
 *                                                                             *
 * Copyright (c) 2006 Altera Corporation, San Jose, California, USA.           *
 * All rights reserved.                                                        *
 *                                                                             *
 * Permission is hereby granted, free of charge, to any person obtaining a     *
 * copy of this software and associated documentation files (the "Software"),  *
 * to deal in the Software without restriction, including without limitation   *
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
 * and/or sell copies of the Software, and to permit persons to whom the       *
 * Software is furnished to do so, subject to the following conditions:        *
 *                                                                             *
 * The above copyright notice and this permission notice shall be included in  *
 * all copies or substantial portions of the Software.                         *
 *                                                                             *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
 * DEALINGS IN THE SOFTWARE.                                                   *
 *                                                                             *
 * This agreement shall be governed in all respects by the laws of the State   *
 * of California and by the laws of the United States of America.              *
 * Altera does not recommend, suggest or require that this reference design    *
 * file be used in conjunction or combination with any other product.          *
 ******************************************************************************/
